// 


/*
 *	Fractals: Fractal analysis algorithms
 */


/*		CRYSTAL IMAGE 0.9.0 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */



#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>





#include "imgbuf.h"
#include "functions.h"
#include "dlgbox.h"
#include "roi.h"
#include "graphdsp.h"
#include "gtkhelpers.h"
#include "fractals.h"
#include "morphop.h"
#include "application.h"
#include "secondwnd.h"


#define MAXDIMPTS 512

double dim_bxsize[MAXDIMPTS];	/* to store the log-log data */
double dim_numboxes[MAXDIMPTS];	/* for display and examination */
int dimpts;
double dim_thresh[MAXDIMPTS];
double dim_dimension[MAXDIMPTS*2];



/* This module provides a series of "notions" for the fractal dimension.
   They have certain limitations and applications. These notions are:

1. Box-counting. Limitation: Any image is thresholded and assumed binary.
2. Minkowsky dimension. Same limitation.
3. Mass dimension: Works on binary _and_ grayscale images, but all
					circles are centered on the image center.
4. Manhattan dimension. Works on binary _and_ grayscale images.

*/


/* Notion 1: Binary box-counting dimension (most basic). A "one"
   is everything between lower and upper threshold.
   ROI is observed - a box is examined only if one of the corners
   falls within the ROI. Caution - an irregular ROI shape may
   influence the F.D. result.
   
   Returns one global parameter for slice z */

fitresult boxdim2d (image_type* img, int z, int bmin, int bmax, float lthresh, float uthresh)
{
int x,y,x1,y1;
int i,dn,bxset,bxtotal, setflg;
float buf;
fitresult fit;
int bx1,bx2,by1,by2;


	fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
	if (slice_is_zero (img,z)) return fit;

	roi_bbox (img, &bx1,&by1,&bx2,&by2);

	i = 0; 
	dn=bmin; if (!dn) dn=1;	
	if (bmax > MIN(bx2-bx1,by2-by1)) bmax = MIN(bx2-bx1,by2-by1);
	
	do
	{
		bxset=0; bxtotal=0;    
		for (y=by1; y<img->ymax; y+=dn)		/* aligns with ROI bbox */
			for (x=bx1; x<img->xmax; x+=dn)
			{
				if ((in_roi(x,y) || in_roi(x+dn-1,y) || in_roi(x,y+dn-1) || in_roi (x+dn-1,y+dn-1)))
				{
					bxtotal++;
					setflg=0;  /* Test if a minimum of one pixel is set */
					y1=0;
					while ((y1<dn) && (y1+y < img->ymax) && (setflg==0))
					{
						x1=0;
						while ((x1<dn) && (x1+x < img->xmax) && (setflg==0))
						{
							if (in_roi(x+x1,y+y1))
							{
								buf=readbuf_flt (img, x+x1, y+y1, z);
								if ((lthresh<=buf) && (buf<=uthresh)) setflg=1;
							}
							x1++;
						}
						y1++; 
					}
					if (setflg != 0) bxset++;
				}
			}
			
		/* We discard any point where the number of counted boxes is zero */

		if (bxset)
		{		
			dim_bxsize[i]=dn; dim_numboxes[i]=bxset; dimpts=i;
			i++;  
		}
		
		dp (1, "Boxdim: Round %d, box size %d, total boxes %d, counted boxes %d\n",
							i,dn,bxtotal,bxset);
		dn +=dn;	/* double box side */
		
	}
	while ((dn <= bmax));
	
	/* Done, perform the log-log fit */
	
	if (i>2)
	{
		fit = loglogfit (dim_bxsize, dim_numboxes, 0,dimpts);
		fit.H = fabs (fit.H);
		dp (1, "Log-log fit: Slope=%5.2f, regression %5.2f\n", fit.H, fit.r);
		dimpts++;	/* Outside, we use dimpts as the number of pts, not as the last pt. */
		return fit;
	}
	else
	{
		dp (1, "Insufficient data to perform fit\n");
		fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
		return fit;
	}
	
}


/* Box-counting II: Perform repeated box-countings with varying threshold
   values. lthresh and uthresh have different meanings here, it's the
   threshold range to cover.
   The result goes into the arrays dim_thresh and dim_dimension.
*/

#define NUMSTEPS   20

void dim_vs_thresh (image_type* img, int z, int bmin, int bmax, float lthresh, float uthresh)
{
int i,imax;
float delta, t;
fitresult dim;


	/* First of all, determine a good delta for the number of steps req'd */
	
	delta = uthresh - lthresh;
	if (delta<=NUMSTEPS)
		delta = 1.0;
   else
      delta = calcstep (lthresh, uthresh, NUMSTEPS);

	/* Simulate a cycle to obtain imax */

	i=0; t = lthresh;
	do
	{
		i++; t+=delta;
	}
	while (t <= uthresh);
	imax = i-1;

	/* Now cycle thrugh the threshold values */

	i=0; t = lthresh;
	do
	{
		dim = boxdim2d (img, z, bmin, bmax, t, uthresh);
		dim_thresh[i] = t;
		dim_dimension[i] = dim.H;
		dim_dimension[i + imax] = dim.r;
		i++; t+=delta;
	}
	while (t <= uthresh);
	dimpts=i-1;

}


void dim_vs_z (image_type* img, int bmin, int bmax, float lthresh, float uthresh)
{
int z;
fitresult dim;

	for (z=0; z<img->zmax; z++)
	{
		dim = boxdim2d (img, z, bmin, bmax, lthresh, uthresh);
		dim_thresh[z] = z;
		dim_dimension[z] = dim.H;
		dim_dimension[z + img->zmax] = dim.r;
	}
	dimpts=img->zmax;

}


/* Notion 2: Manhattan dimension. The Manhattan dimension is
   an extension to the box-counting dimension. 
   Each box contains a number of pixels, interpreted as elevations.
   The differences in elevation are counted and plotted over the number
   of pixels (side length ^2) of the box. The name "Manhattan" cones from a
   variation of this function where each pixel is interpreted as a skyscraper
   of the pixel value's height. The area is the surface (sides!) of the skyscrapers.
   The version used here is based on two triangles defined by four neighboring pixels.

	This is clearly a grayscale algorithm and does not require a thresholded image.
	The GUI's threshold sliders do not have an effect on this algorithm.
	It works on a single slice. Starting from the single-pixel level, squares are
	doubled in size, and bmin and bmax determine the log2 fit range. For example
	with bmin=3 and bmax=8, the data points are generated from the lowest scale
	but only considered from a box size of 4x4 and up to 128x128. The oorflag
	determines if zero values are considered part of the feature or part of the background.
	
   
	TODO: Cannot yet handle ROIs. We might need a mask to handle ROI.
*/


float triangle_area (float ax, float ay, float az, float bx, float by, float bz, float cx, float cy, float cz)
{
float side_a, side_b, side_c, s;

	side_a = sqrt (SQR(ax-bx) + SQR(ay-by) + SQR(az-bz));
	side_b = sqrt (SQR(ax-cx) + SQR(ay-cy) + SQR(az-cz));
	side_c = sqrt (SQR(bx-cx) + SQR(by-cy) + SQR(bz-cz));
	s = (side_a+side_b+side_c)*0.5;
	
	return sqrt (s*(s-side_a)*(s-side_b)*(s-side_c));

}


fitresult manhattan_boxdim (image_type* img, int z, int bmin, int bmax, int oorflg)
{
int x,y,x1,y1,xm,ym, xr,yr;
int i,j,bxarea, scale;
float bxmass;
fitresult fit;
int bx1,bx2,by1,by2;
float *p1, *p2;
unsigned char *mask1, *mask2;
float z00, z01, z10, z11;


	fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0; 
	xm = img->xmax; ym = img->ymax; xr=xm; yr=ym;
	if (slice_is_zero (img,z)) return fit;

	p1 = calloc (img->xmax * img->ymax, sizeof (float));
	p2 = calloc (img->xmax * img->ymax, sizeof (float));
	mask1 = calloc (img->xmax * img->ymax, sizeof (char));
	mask2 = calloc (img->xmax * img->ymax, sizeof (char));
	if ((p1==NULL) || (p2==NULL) || (mask1==NULL) || (mask2==NULL)) fatal_memerr();
	
	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
			p1[x + xm*y] = readbuf_flt (img, x,y,z);
			p2[x+xm*y] = p1[x + xm*y];
			if (in_roi (x,y)) { mask1[x+xm*y]=1; mask2[x+xm*y]=1; }
		}


	roi_bbox (img, &bx1,&by1,&bx2,&by2);

	i = 0; j=0; scale=1;

	/* if (bmax > MIN(bx2-bx1,by2-by1)) bmax = MIN(bx2-bx1,by2-by1); */
	do
	{
	
		/* Sum up the triangulated (tesselated?) area, since we have
		   the 4 points perform averaging right away. */
	
		i++; bxmass  =0.0; y1=0; bxarea=0;
		for (y=0; y<yr; y+=2)
		{
			x1=0; 
			for (x=0; x<xr; x+=2)
			{
				z00 = p1[x  +xm*y];
				z01 = p1[x  +xm*(y+1)];
				z10 = p1[x+1+xm*y];
				z11 = p1[x+1+xm*(y+1)];
				if (oorflg && ((z00==0.0) || (z01==0.0) || (z10==0.0) || (z11==0.0)))
				{
					p2[x1 + xm*y1] = 0.0;	/* Propagate areas touching a zero-OOR pixel */
				}
				else
				{
					/* Each square consists of 2 triangles, which are planar */
					if (mask1[x+xm*y] && mask1[x+1+xm*y] && mask1[x+xm*(y+1)] && mask1[x+1+xm*(y+1)])
					{
						bxmass += triangle_area (x*scale,y*scale,z00, (x+1)*scale,y*scale,z10, x*scale,(y+1)*scale,z01);
						bxmass += triangle_area ((x+1)*scale,(y+1)*scale,z11, (x+1)*scale,y*scale,z10, x*scale,(y+1)*scale,z01);
						bxarea++;
						mask2[x1+xm*y1]=1;
					}
					else
						mask2[x1+xm*y1]=0;
					p2[x1 + xm*y1] = 0.25*(z00+z01+z10+z11);
				}
				x1++;
			}
			y1++;
		}

		/* copy the scaled-down version into the primary buffer */
		
		xr = xr>>1; yr=yr>>1;
		for (y=0; y<yr; y++)
			for (x=0; x<xr; x++)
			{
				p1[x+xm*y] = p2[x+xm*y];
				mask1 [x+xm*y] = mask2[x+xm*y];
			}
		
			
		/* We discard any point where the box mass is zero */

		if ((bxarea) && (bxmass) && (i>=bmin) && (i<=bmax))
		{		
			dim_bxsize[j]=scale*scale; dim_numboxes[j]=bxmass; dimpts=j;
			j++;  
		}
		
		dp (1, "Blanket Dimension: Round %d, box area %d, box area %f\n",
							i,scale*scale,bxmass);
		
		scale = scale<<1;
	}
	while ((i <= bmax));
	
	free (p1); free (p2);
	free (mask1); free (mask2);
	
	/* Done, perform the log-log fit */
	
	if (j>2)
	{
		fit = loglogfit (dim_bxsize, dim_numboxes, 0,dimpts);
		fit.H = fabs (fit.H);
		dp (1, "Log-log fit: Slope=%5.2f, regression %5.2f\n", fit.H, fit.r);
		dimpts++;	/* Outside, we use dimpts as the number of pts, not as the last pt. */
		return fit;
	}
	else
	{
		dp (1, "Insufficient data to perform fit\n");
		fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
		return fit;
	}
	
}



/* Notion 3: Minkowsky dimension, also called "capacitor dimension".
	The source is a binary image, ideally the outline of an object.
	This outline is repeatedly iteratively dilated (equivalent to
	individual dilations with increasing dilation kernels) and the
	gained mass is measured. This mass gain, log-log over the dilation 
	kernel diam, yields a slope s1, which is related to the dimension D
	by D = 2-s1. If we divide the mass gain by the dilation kernel 
	diameter, we get a slope s2 with d = 1-s2.
	
	Strictly, this dimension is valid only for binary images and for
	outlines of the objects. Here we have two modifications to
	generalize the algorithm:
	- The original image is subtracted from the dilated image, resulting
	  in the outline
	- we also allow grayscale images, because our morph ops are generalized
	  anyway. In this case, the mass gain is the integral over the gray values.

	The input values bmin and bmax are the radii over which the dilation
	is measured. Only values falling within lthersh..uthresh are considered.
*/



fitresult sausagedim (image_type* img, int z, int bmin, int bmax, float lthresh, float uthresh)
{
int x,y;
int i,j,radius;
float buf, mass;
image_type srcimg, dilimg;
float *p1, *p2;
fitresult fit;
int bx1,bx2,by1,by2;
long idx;


	fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
	if (slice_is_zero (img,z)) return fit;

	roi_bbox (img, &bx1,&by1,&bx2,&by2);

	if (allocate_image (&srcimg, B_FLOAT, img->xmax, img->ymax, 1)==-1) 
		fatal_memerr();
	if (allocate_image (&dilimg, B_FLOAT, img->xmax, img->ymax, 1)==-1) 
		fatal_memerr();

	p1 = srcimg.data;
	p2 = dilimg.data;

	/* Clone the current slice and copy into original & dilatable image */

	for (y=0; y<img->ymax; y++)
	{
		for (x=0; x<img->xmax; x++)
		{
			if (in_roi(x,y))		/* Not in ROI or not in thresh remains zero */
			{
				buf = readbuf_flt (img,x,y,z);
				if ((buf>=lthresh) && (buf<=uthresh))
				{
					idx = x + img->xmax * y;
					p1[idx]=buf; p2[idx]=buf;	/* Set both images (initial diff is zero) */
				}
			}
		}
	}

	/* Perform the iterative dilations. COmpute mass from difference
	   between dilated and original image */


	j=0;
	for (i=0; i<bmax; i++)		/* Iterations */
	{
		morph_img (&dilimg, M_DILATE, 0, 1);

		/* Compute mass; Difference between dilated and original image */

		mass = 0.0;
		for (y=0; y<img->ymax; y++)
			for (x=0; x<img->xmax; x++)
			{
				idx = x + y*img->xmax;
				mass += p2[idx] - p1[idx];
			}
		
		radius = 2*i+1;	/* Dilation kernel radius */
		
		if (i>=bmin)
		{
			dim_bxsize[j]=radius; dim_numboxes[j]=mass; dimpts=j;
			j++;  
			dp (1, "*");
		}
		else
			dp (1, " ");
			
		dp (1, " Minkowsky: Iteration %d: Diameter %d; mass %8.1f\n",i,radius,mass);
		
	}

	/* Done, show the final difference and perform the log-log fit */
	
	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
			idx = x + y*img->xmax;
			p2[idx] -= p1[idx];
		}

	if (!macrorun)
	{
		make_minmax (&dilimg);
		secondary_display (&dilimg,0,"Final difference image");
	}

	freebuf (&srcimg);
	freebuf (&dilimg);
	
	if (i>2)
	{
		fit = loglogfit (dim_bxsize, dim_numboxes, 0,dimpts);
		fit.H = fabs (fit.H);
		dp (1, "Log-log fit: Slope=%5.2f, regression %5.2f\n", fit.H, fit.r);
		dimpts++;	/* Outside, we use dimpts as the number of pts, not as the last pt. */
		return fit;
	}
	else
	{
		dp (1, "Insufficient data to perform fit\n");
		fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
		return fit;
	}


}




void sdim_vs_z (image_type* img, int bmin, int bmax, float lthresh, float uthresh)
{
int z;
fitresult dim;

	for (z=0; z<img->zmax; z++)
	{
		dim = sausagedim (img, z, bmin, bmax, lthresh, uthresh);
		dim_thresh[z] = z;
		dim_dimension[z] = 2.0-dim.H;	/* Immediately convert slope to dimension */
		dim_dimension[z + img->zmax] = dim.r;
	}
	dimpts=img->zmax;

}


/* Notion 4: Mass dimension

   The second dimension needs a seed point. We compute the gray value 
   content of ever increasing circles and plot it over the area.

	The advantage of both notions is obviously the independence of a threshold.

	TODO: Cannot yet handle ROIs. We might need a mask to handle ROI.
*/



fitresult mass_dimension (image_type* img, int z, int bmin, int bmax, int oorflag)
{
int x,y,xm,ym;
int j,radius;
float buf, mass;
fitresult fit;
int bx1,bx2,by1,by2;


	fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
	if (slice_is_zero (img,z)) return fit;

	roi_bbox (img, &bx1,&by1,&bx2,&by2);

	xm=img->xmax/2; ym=img->ymax/2;		/* Warning: No safeguard that circle stays within image area */

										/* Warning - we increment the radius by one, this gives us an
											insane amount of data for large circles. Should we double
											the radius? Make it user-selectable? */


	j=0;
	for (radius=bmin; radius<=bmax; radius++)		/* Iterations */
	{

		/* Compute mass; Pixel mass underneath circle of radius r. 
			We cover a square centered on (xm,ym) and test for 
			circularity later */

		mass = 0.0;
		for (y=MAX(0,ym-radius); y<MIN(img->ymax,ym+radius); y++)
			for (x=MAX(0,xm-radius); x<MIN(img->xmax,xm+radius); x++)
			{
				if ( (SQR(x-xm)+SQR(y-ym)) < SQR(radius) )
				{

					buf = readbuf_flt (img,x,y,z);
					if (!oorflag || (fabs(buf) < 1e-12))
					{
						mass += buf;
					}
				}
			}
		
		if ((j < MAXDIMPTS) && (radius>0) && (mass>0))	/* Prevent zero-mass data which fail the LOG */
		{
			dim_bxsize[j]=radius; 
			dim_numboxes[j]=mass; 
			dimpts=j;
			j++;
			dp (1, "*");
		}
		else
			dp (1, " ");
			
		dp (1, " Mass dimension: Iteration %d: Diameter %d; mass %8.1f\n",j,radius,mass);
		if (j >= MAXDIMPTS)	 dp (1, "!!! Too many data points !!! \n");
	}

	if (j>2)
	{
		fit = loglogfit (dim_bxsize, dim_numboxes, 0,dimpts);
		fit.H = fabs (fit.H);
		dp (1, "Log-log fit: Slope=%5.2f, regression %5.2f\n", fit.H, fit.r);
		dimpts++;	/* Outside, we use dimpts as the number of pts, not as the last pt. */
		return fit;
	}
	else
	{
		dp (1, "Insufficient data to perform fit\n");
		fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
		return fit;
	}


}



/*************************************************

	Dispersion analysis:. Another method to compute
	the Hurst exponent.
	Dispersion analysis looks at the effect of averaging
	neighboring points. The SD of each scaled-down image with
	increasing bin size is drawn over the bin size,
	and a log-log line is fitted. The slope of this line 
	equals H-1.
	DISPERSION ANALYSIS WORKS ONLY ON FRACTIONAL BROWNIAN NOISE, IF AT ALL.

*/



fitresult dispersion_boxdim (image_type* img, int z, int bmin, int bmax, int oorflg)
{
int x,y,x1,y1,xm,ym, xr,yr;
int i,j, bxarea, scale, Sx, Sxx;
float avg, stdev;
fitresult fit;
int bx1,bx2,by1,by2;
float *p1, *p2;
unsigned char *mask1, *mask2;
float z00, z01, z10, z11;


	fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0; 
	xm = img->xmax; ym = img->ymax; xr=xm; yr=ym;
	if (slice_is_zero (img,z)) return fit;

	p1 = calloc (img->xmax * img->ymax, sizeof (float));
	p2 = calloc (img->xmax * img->ymax, sizeof (float));
	mask1 = calloc (img->xmax * img->ymax, sizeof (char));
	mask2 = calloc (img->xmax * img->ymax, sizeof (char));
	if ((p1==NULL) || (p2==NULL) || (mask1==NULL) || (mask2==NULL)) fatal_memerr();
	
	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
	{
		p1[x + xm*y] = readbuf_flt (img, x,y,z);
		p2[x+xm*y] = p1[x + xm*y];
		if (in_roi (x,y)) { mask1[x+xm*y]=1; mask2[x+xm*y]=1; }
	}


	roi_bbox (img, &bx1,&by1,&bx2,&by2);

	i = 0; j=0; scale=1;

	/* if (bmax > MIN(bx2-bx1,by2-by1)) bmax = MIN(bx2-bx1,by2-by1); */
	do
	{
	
		i++; bxarea=0; Sx=0; Sxx=0; avg=0; stdev=0;
		y1=0;
		for (y=0; y<yr; y+=2)
		{
			x1=0; 
			for (x=0; x<xr; x+=2)
			{
				z00 = p1[x  +xm*y];
				z01 = p1[x  +xm*(y+1)];
				z10 = p1[x+1+xm*y];
				z11 = p1[x+1+xm*(y+1)];
				if (oorflg && ((z00==0.0) || (z01==0.0) || (z10==0.0) || (z11==0.0)))
				{
					p2[x1 + xm*y1] = 0.0;	/* Propagate areas touching a zero-OOR pixel */
				}
				else
				{
					/* Add up for stdev computation */
					if (mask1[x+xm*y] && mask1[x+1+xm*y] && mask1[x+xm*(y+1)] && mask1[x+1+xm*(y+1)])
					{
						Sx += z00; Sxx += z00*z00;
						bxarea++;
						mask2[x1+xm*y1]=1;
					}
					else
						mask2[x1+xm*y1]=0;

					p2[x1 + xm*y1] = 0.25*(z00+z01+z10+z11);
				}
				x1++;
			}
			y1++;
		}

		/* Compute stdev at this scale */

		avg=Sx/bxarea; stdev=SDEV(Sx, Sxx, bxarea);

		/* copy the scaled-down version into the primary buffer */
		
		xr = xr>>1; yr=yr>>1;
		for (y=0; y<yr; y++)
			for (x=0; x<xr; x++)
		{
			p1[x+xm*y] = p2[x+xm*y];
			mask1 [x+xm*y] = mask2[x+xm*y];
		}
		
			
		/* We discard any point where the box mass is zero */

		if ((bxarea) && (i>=bmin) && (i<=bmax))
		{		
			dim_bxsize[j]=scale*scale; dim_numboxes[j]=stdev; dimpts=j;
			j++;  
			dp (1, "* ");
		}
		else
			dp (1, " ");
		
		dp (1, "Dispersion Dimension: Round %d, box area %d, Stdev %f\n",
			i,scale*scale,stdev);
		
		scale = scale<<1;
	}
	while ((i <= bmax));
	
	free (p1); free (p2);
	free (mask1); free (mask2);
	
	/* Done, perform the log-log fit */
	
	if (j>2)
	{
		fit = loglogfit (dim_bxsize, dim_numboxes, 0,dimpts);
		fit.H = fabs (fit.H);
		dp (1, "Log-log fit: Slope=%5.2f, regression %5.2f, H=%5.2f\n", fit.H, fit.r,1.0-fit.H);
		dimpts++;	/* Outside, we use dimpts as the number of pts, not as the last pt. */
		fit.H=1.0-fit.H;	/* Convert slope into Hurst exponent */
		return fit;
	}
	else
	{
		dp (1, "Insufficient data to perform fit\n");
		fit.H=0; fit.intcpt=0; fit.r=0; dimpts=0;
		return fit;
	}
	
}





/**********************************************************

	LACUNARITY
	Lacunarity is a function of the local mass, like Hoelder.
	For each pixel, we compute the sum of pixels in a moving
	window of given radius. With this, we create a second
	image with the local mass, and the lacunarity L is defined
	as the variance divided by the squared mean of that
	mass image.

*************************************************************/

/* Read buffer with mirror boundary conditions */

double lacu_readbuf (image_type* img, int x, int y, int z)
{
	if (x<0) x=abs(x); else if (x>=img->xmax) x=(2*img->xmax)-x-1;
	if (y<0) y=abs(y); else if (y>=img->ymax) y=(2*img->ymax)-y-1;
	return readbuf_flt (img, x, y, z);
}


double lacunarity (image_type* img, int slice, int radius)
{
int x,y,x1,y1,i,j,k,n,c, firstcol;
int bx1,bx2,by1,by2;
double buf, colmass, mass, Sx, Sxx, cnt, mean, var;



	Sx=0.0; Sxx=0.0; cnt=0.0; c=radius;
	roi_bbox (img, &bx1, &by1, &bx2, &by2); 			/* Observe ROI */


	for (y=by1; y<=by2; y++)			/* For each pixel of the current image ... */
	{
		firstcol=1;
		for (x=bx1; x<bx2; x++)
		{
			/* Scan the local neighborhood to get mass. To speed up the process,
				we obtain S for the entire square for the first column only (firstcol==1).
				In subsequent columns, we subtract the leftmost column (colmass)
				and add only the rightmost column. */

			if (firstcol)		/* First column in the present line. Compute the full square */
			{
				mass = 0.0; colmass = 0.0;
				for (j=-c; j<=c; j++)
					for (i=-c; i<=c; i++)
					{
						buf = lacu_readbuf (img, x+i, y+j, slice);
						mass += buf;
						if (j==-c) colmass+=buf;		/* Leftmost column */
					}
				firstcol=0;
			}
			else			/* Subsequent columns. Subtract left column, add right column */
			{
				mass -= colmass; colmass=0.0;
				for (j=-c; j<=c; j++)
				{
					mass += lacu_readbuf (img, x+c, y+j, slice);
					colmass += lacu_readbuf (img, x-c, y+j, slice);
				}
			}
			
			/* Something unusual. To use the above trick with the left/right columns, we need
				a rectangular ROI. To honor polygonal ROIs, we (wastefully) compute the above
				even for out-of-ROI pixels (few as they'll be) and restrict the computation
				of Sxx and Sxx to in_roi pixels. */

			if (in_roi (x,y))
			{
				/* Keep track of Sx and Sxx to compute the moments */

				Sx += mass;
				Sxx += mass*mass;
				cnt += 1;
			}
		}
	}

	/* Done with the pass over the image, compute L(r) */

	if (F_ISZERO(Sx))			/* Zero mean, infinite lacunarity? */
	{
		dp (2, "Zero mean in lacunarity computation for r=%d\n",radius);
		return 0.0;
	}
	
	mean = Sx/cnt;
	var = (Sxx-Sx*mean)/(cnt-1.0);  	/* from (1/N)(Sxx - Sx^2/N) */
	buf = 1.0 + var/(mean*mean);		/* Lacunarity */
	dp (2, "Computed L(r) for r=%d, lacunarity is %f\n", radius, buf);
	return	buf;

}





/*********************************************

	The GUI

**********************************************/


GtkWidget *boxdim_wnd, *dim_label7;
GtkAdjustment *dim_lthresh_adj, *dim_uthresh_adj, *dim_boxminsize_adj, *dim_boxmaxsize_adj;
int dim_function;
int dim_2d3d;
image_type* dim_img;

#define ADDRPT(X)		strncat (rpt,X,32700)

void on_dim_okbutton_clicked  (GtkButton *button, gpointer user_data)
{
int bmin, bmax,i;
float uthresh, lthresh;
fitresult fit;
double buf;
char s[1024];
char rpt[32768];

	rpt[0]=0;
	if (dim_function==0) sprintf (s, "Box-counting\n");
	else if (dim_function==1) sprintf (s, "Dimension vs threshold\n");
	else if (dim_function==2) sprintf (s, "Dimension over stack\n");
	else if (dim_function==3) sprintf (s, "Minkowsky Dimension\n");
	else if (dim_function==4) sprintf (s, "Minkowsky dimension over stack\n");
	else if (dim_function==5) sprintf (s, "Mass dimension\n");
	else if (dim_function==6) sprintf (s, "Manhattan box-counting dimension\n");
	else if (dim_function==7) sprintf (s, "Dispersion box-counting dimension\n");
	else if (dim_function==8) sprintf (s, "Lacunarity L(r)\n");
	ADDRPT (s);

	bmin = dim_boxminsize_adj->value;
	bmax = dim_boxmaxsize_adj->value;
	lthresh = dim_lthresh_adj->value;
	uthresh = dim_uthresh_adj->value;

	if (dim_function==8)
		sprintf (s, "Radius range %d...%d\nThreshold range %8.2f to %8.2f\n",bmin,bmax,lthresh,uthresh);
	else if ((dim_function==3) || (dim_function==4))
		sprintf (s, "Dilation size %d...%d\nThreshold range %8.2f to %8.2f\n",bmin,bmax,lthresh,uthresh);
	else
		sprintf (s, "Box size %d...%d\nThreshold range %8.2f to %8.2f\n",bmin,bmax,lthresh,uthresh);
	ADDRPT(s);

	if (dim_function==0)
	{
		fit = boxdim2d (dim_img, cur_slice, bmin,bmax, lthresh,uthresh);
		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\nSlope = %5.2f, r = %5.2f\n",fit.H, fit.r);
			ADDRPT(s);
			sprintf (s, "\nDimension D = %5.2f\n",fabs(fit.H));
			ADDRPT(s);
			ShowLongMessage(rpt,"Box-counting results");
		}
	}
	else if (dim_function==1)
	{
		dim_vs_thresh (dim_img, cur_slice, bmin,bmax, lthresh,uthresh);
		multi_graphme_d (dim_thresh, dim_dimension, dimpts,2);
		for (i=0; i<dimpts; i++)
		{
			sprintf (s, "T=%8.2f, D=%5.2f, r=%5.2f\n",dim_thresh[i],dim_dimension[i],dim_dimension[i+dimpts]);
			ADDRPT(s);
		}
		ShowLongMessage(rpt,"Box-counting results");
	}
	else if (dim_function==2)
	{
		dim_vs_z (dim_img, bmin,bmax, lthresh,uthresh);
		multi_graphme_d (dim_thresh, dim_dimension, dimpts,2);
		for (i=0; i<dimpts; i++)
		{
			sprintf (s, "Z=%d, D=%5.2f, r=%5.2f\n",(int)dim_thresh[i],dim_dimension[i],dim_dimension[i+dimpts]);
			ADDRPT(s);
		}
		ShowLongMessage(rpt,"Box-counting results");
	}
	else if (dim_function==3)
	{
		fit = sausagedim (dim_img, cur_slice, bmin,bmax, lthresh,uthresh);
		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\nSlope = %5.2f, r = %5.2f\n",fit.H, fit.r);
			ADDRPT(s);
			sprintf (s, "\nDimension D = 2-S = %5.2f\n",2.0-fit.H);
			ADDRPT(s);
			ShowLongMessage(rpt,"Minkowsky Dimension results");
		}
	}
	else if (dim_function==4)
	{
		sdim_vs_z (dim_img, bmin,bmax, lthresh,uthresh);
		multi_graphme_d (dim_thresh, dim_dimension, dimpts,2);
		for (i=0; i<dimpts; i++)
		{
			sprintf (s, "Z=%d, D=%5.2f, r=%5.2f\n",(int)dim_thresh[i],dim_dimension[i],dim_dimension[i+dimpts]);
			ADDRPT(s);
		}
		ShowLongMessage(rpt,"Minkowsky dimension results");
	}
	else if (dim_function==5)
	{
		fit = mass_dimension (dim_img, cur_slice, bmin,bmax,0);
		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\nSlope = %5.2f, r = %5.2f, n=%d\n",fit.H, fit.r, dimpts);
			ADDRPT(s);
			sprintf (s, "\nDimension D = S = %5.2f\n",fabs(fit.H));
			ADDRPT(s);
			ShowLongMessage(rpt,"Mass dimension results");
		}
	}
	else if (dim_function==6)
	{
		fit = manhattan_boxdim (dim_img, cur_slice, bmin,bmax,0);
		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\nSlope = %5.2f, r = %5.2f, n=%d\n",fit.H, fit.r, dimpts);
			ADDRPT(s);
			sprintf (s, "\nDimension D = 2-S = %5.2f\n",2.0-fabs(fit.H));
			ADDRPT(s);
			ShowLongMessage(rpt,"Manhattan Box-counting results");
		}
	}
	else if (dim_function==7)
	{
		fit = dispersion_boxdim (dim_img, cur_slice, bmin,bmax,0);
		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\nHurst = %5.2f, r = %5.2f, n=%d\n",fit.H, fit.r, dimpts);
			ADDRPT(s);
			sprintf (s, "\nDimension D = 2-S = %5.2f\n",2.0-fabs(fit.H));
			ADDRPT(s);
			ShowLongMessage(rpt,"Dispersion Box-counting results");
		}
	}
	else if (dim_function==8)
	{
		dimpts=0;
		for (i=bmin; i<=bmax; i++)
		{
			buf = lacunarity (dim_img, cur_slice, i);
			dim_bxsize[dimpts]=i;
			dim_numboxes[dimpts]=buf;
			dimpts++;
			indicate_progress ( (float)dimpts/(1.0+bmax-bmin));
		}

		reset_progress();

		if (dimpts)
		{
			graphme_d (dim_bxsize, dim_numboxes, dimpts);
			sprintf (s, "\n r       L(r)\n-------------\n");
			ADDRPT(s);
			for (i=0; i<dimpts; i++)
			{
				sprintf (s, "%3.0f   %7.5f\n", dim_bxsize[i], dim_numboxes[i]);
				ADDRPT(s);
			}
			ShowLongMessage(rpt,"Tabular lacunarity results");
		}
	}

	gtk_widget_destroy (boxdim_wnd);

}

void on_dim_cancelbutton_clicked  (GtkButton *button, gpointer user_data)
{

	gtk_widget_destroy (boxdim_wnd);
}

void dim_set_dim_menu (GtkWidget *widget, gpointer user_data)
{

	dim_function = *(int*) user_data;
	
}

void on_dim_2d3dbutton_clicked (GtkWidget *widget, gpointer user_data)
{

	dim_2d3d = !dim_2d3d;
	if (dim_2d3d)
		gtk_label_set_text (GTK_LABEL(dim_label7), "3D");
	else
		gtk_label_set_text (GTK_LABEL(dim_label7), "2D");

	
}



void create_boxdim_wnd (image_type* img)
{
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *table1;
  GtkWidget *dim_uthresh;
  GtkWidget *dim_lthresh;
  GtkWidget *dim_boxmaxsize;
  GtkWidget *dim_boxminsize;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *dim_2d3dbutton;
  GtkWidget *dim_optionmenu;
  GtkWidget *dim_optionmenu_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *dim_okbutton;
  GtkWidget *dim_cancelbutton;
  GtkWidget *hseparator1;
  GtkWidget *hseparator2;
  float min, max;
  int boxmax;
  
	dim_img = img;
	imgminmax (img, &min, &max);
	boxmax = MIN(img->xmax, img->ymax)/2;
	dim_2d3d=0;
	dim_function=0;

	boxdim_wnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (boxdim_wnd), "boxdim_wnd", boxdim_wnd);
	gtk_container_set_border_width (GTK_CONTAINER (boxdim_wnd), 3);
	gtk_window_set_title (GTK_WINDOW (boxdim_wnd), "Box-counting Dimension");
	gtk_window_set_modal (GTK_WINDOW (boxdim_wnd), TRUE);

	vbox1 = vbox_in_window (boxdim_wnd,"dim_vbox1");
	label1 = MakeLabel (boxdim_wnd,vbox1,"Fractal Dimension Estimator","label1",-1,-1);
	table1 = MakeTable (boxdim_wnd,vbox1,"dim_table1",2,8);

	label2 = MakeLabel (boxdim_wnd,table1,"Dimension","label2",0,0);
	label3 = MakeLabel (boxdim_wnd,table1,"Lower threshold","label3",0,1);
	label4 = MakeLabel (boxdim_wnd,table1,"Upper threshold","label4",0,2);
	label5 = MakeLabel (boxdim_wnd,table1,"Minimum box size","label5",0,3);
	label6 = MakeLabel (boxdim_wnd,table1,"Maximum box size","label6",0,4);
	dim_label7 = MakeLabel (boxdim_wnd,table1,"2D","label7",0,5);

	dim_lthresh_adj = GTK_ADJUSTMENT (gtk_adjustment_new (min, min, max, 1, 10, 0));
	dim_lthresh = MakeHScale (dim_lthresh_adj,boxdim_wnd,table1,"dim_lthresh",1,1);
	
	dim_uthresh_adj = GTK_ADJUSTMENT (gtk_adjustment_new (max, min, max, 1, 10, 0));
	dim_uthresh = MakeHScale (dim_uthresh_adj,boxdim_wnd,table1,"dim_uthresh",1,2);
	
	dim_boxminsize_adj = GTK_ADJUSTMENT (gtk_adjustment_new (1, 1, boxmax, 1, 10, 0));
	dim_boxminsize = MakeHScale (dim_boxminsize_adj,boxdim_wnd,table1,"dim_minsize",1,3);
	gtk_scale_set_digits (GTK_SCALE (dim_boxminsize), 0);

	dim_boxmaxsize_adj = GTK_ADJUSTMENT (gtk_adjustment_new (10, 1, boxmax, 1, 10, 0));
	dim_boxmaxsize = MakeHScale (dim_boxmaxsize_adj,boxdim_wnd,table1,"dim_maxsize",1,4);
	gtk_scale_set_digits (GTK_SCALE (dim_boxmaxsize), 0);

	dim_2d3dbutton = Togglebutton_in_table (boxdim_wnd,table1,"2D / 3D","dim_2d3dbutton",1,5,on_dim_2d3dbutton_clicked);

	dim_optionmenu = Make_Optionmenu (boxdim_wnd,table1,"dim_optionmenu",1,0);
	gtk_container_set_border_width (GTK_CONTAINER (dim_optionmenu), 1);
	dim_optionmenu_menu = gtk_menu_new ();
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Box-counting dimension", (gpointer) &menuopt0);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Dimension vs threshold", (gpointer) &menuopt1);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Dimension vs Z-slice", (gpointer) &menuopt2);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Minkowsky dimension", (gpointer) &menuopt3);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Minkowsky dimension vs z-slice", (gpointer) &menuopt4);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Mass dimension", (gpointer) &menuopt5);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Manhattan Box-conting dimension", (gpointer) &menuopt6);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Dispersion Box-conting dimension", (gpointer) &menuopt7);
	glade_menuitem = Option_menuitem (dim_optionmenu_menu, GTK_SIGNAL_FUNC(dim_set_dim_menu), "Lacunarity", (gpointer) &menuopt8);	
	gtk_option_menu_set_menu (GTK_OPTION_MENU (dim_optionmenu), dim_optionmenu_menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (dim_optionmenu), 0);
	
	hseparator1 = MakeHsep (boxdim_wnd,table1,"hsep1",0,6);
	hseparator2 = MakeHsep (boxdim_wnd,table1,"hsep2",1,6);

	dim_okbutton = Button_in_table (boxdim_wnd,table1,"OK","dim_okbutton",0,7,on_dim_okbutton_clicked);
	dim_cancelbutton = Button_in_table (boxdim_wnd,table1,"Cancel","dim_cancelbutton",1,7,on_dim_cancelbutton_clicked);
	gtk_signal_connect (GTK_OBJECT (boxdim_wnd), "destroy",
                      GTK_SIGNAL_FUNC (on_dim_cancelbutton_clicked), boxdim_wnd);

	gtk_widget_show (boxdim_wnd);
}


void boxdim_main ()
{

	create_boxdim_wnd (&mainimg);
	
}



/*********************************************************

	Rose Plot. This needs to be performed on a FFT!
	
	We sample the amplitudes along several angles and
	compute the 1/f trend (log amplitude over log freq)
	where the slope s is related to the Hurst coefficient
	by s = 2H+1 and to the dimension by s = 2D-1.
	Each value is plotted over its angle in polar coords

**********************************************************/

/* TODO: Additional info can be gained from the magnitude anisotropy, i.e.,
	the intercept of the fit in polar coordinates */

#define ROSE_MAX	96


/* This is the GUI-controlled version */

void rose_plot (image_type* img, int z, int freqstart, int freqend, int rose_max)
{
image_type fftimg;
float ang, real, imag;
float x,y,dx,dy;
int xm,ym,len,i,j,k;
double angles[ROSE_MAX+1];
double hurst[ROSE_MAX+1];
float xval[ROSE_MAX+2];
float yval[ROSE_MAX+2];
double* frq; double* amp;
double Hmax, Hmin, Havg, Hratio;
fitresult fit;
char resultbuf[256];


	/* First, extract the selected slice and perform the FFT.
		To avoid overwriting img with the FFT data, we need to clone the slice */
	
	if ( allocate_image (&fftimg, img->imgtype, img->xmax, img->ymax, 1) == -1)
		fatal_memerr();
	copybuf_z (img, &fftimg, z,z);
	fft2d (&fftimg, 0);

	xm = fftimg.xmax/2; ym = fftimg.ymax/2;
	len = (MIN(xm,ym)-1);							/* Highest possible frequency index */
	freqend = MIN(freqend,len);						/* restrict requested end frequency */
	rose_max = MIN(rose_max, ROSE_MAX); 			/* limit to 96 points */
	
	frq = calloc (len+20, sizeof (double));
	amp = calloc (len+20, sizeof (double));
	if ((frq==NULL) || (amp==NULL)) fatal_memerr();
	
	k=0; Hmax=Hmin=Havg=0.0;
	for (i=0; i<=360; i+=360/rose_max)
	{
		ang = (i+10)*PI/180.0;
		dx = cos(ang); dy = sin(ang); x=xm; y=ym;
		for (j=1; j<len; j++)
		{
			x+=dx; y+=dy;
			frq[j-1]=log10(j);
			real = ireadbuf (&fftimg, x,y,0,intp_pref);
			imag = ireadbuf (&fftimg, x,y,1,intp_pref);
			amp[j-1]=SAFELOG10( sqrt (SQR(real)+SQR(imag)) );
		}

		fit = lsqfit (frq,amp,freqstart, freqend);
		dp (1, "Rose plot: angle %d, H=%f, r=%f\n",i,fit.H, fit.r);
		
		angles[k]=i; hurst[k]=0.5*(-fit.H+1);	/* This is really the dimension */
		angles[k]=i; hurst[k]=-fit.H;	/* Slope beta */

		if (i==0) 
		{
			Havg=Hmin=Hmax=hurst[k]; 
		}
		else
		{
			Hmax = MAX (Hmax, hurst[k]);
			Hmin = MIN (Hmin, hurst[k]);
			Havg += hurst[k];
		}

		xval[k] = hurst[k]*cos(ang);	/* We convert amp over frq data to polar x-y data */
		yval[k] = hurst[k]*sin(ang);
		k++;
	}

	Havg /= (k-1); Hratio = Hmax/Hmin;
	sprintf (resultbuf, "Rose: Average Hurst exponent: %5.2f. Anisotropy %5.2f",Havg,Hratio);
	dp (1, resultbuf); dp (1,"\n");
	statusbar_display (resultbuf);

	graphme (xval, yval, k);
	free (frq); free (amp);

	freebuf (&fftimg);

}

/* Similar code, but for the macro interpreter. Store result in user[] array */


float rose_plot_for_macro (image_type* fftimg, int startpt, int numrays, int maxlen)
{
float ang, real, imag;
float x,y,dx,dy;
int xm,ym,len,i,j,j1,k,freqend;
double angles[numrays+1];
double hurst[numrays+1];
double* frq; double* amp;
double Hmax, Hmin, Havg, Hratio;
fitresult fit;

	if ((fftimg->imgtype!=B_FLOAT) || (fftimg->zmax!=2))
		return -1; /* Error code: This is not a frequency image */

	xm = fftimg->xmax/2; ym = fftimg->ymax/2;
	len = (MIN(xm,ym)-1);							/* Highest possible frequency index */
	freqend = MIN(startpt+maxlen,len);				/* restrict requested end frequency */
	numrays = MIN(numrays,80);				/* Limit to 80 points to not exceed the user[] array */
	
	frq = calloc (len+20, sizeof (double));
	amp = calloc (len+20, sizeof (double));
	if ((frq==NULL) || (amp==NULL)) fatal_memerr();
	
	k=0; Hmax=Hmin=Havg=0.0;
	for (i=0; i<=360; i+=360/numrays)
	{
		ang = (i+10)*PI/180.0;
		dx = cos(ang); dy = sin(ang); x=xm; y=ym; j1=0;
		for (j=1; j<freqend; j++)
		{
			x+=dx; y+=dy;
			if (j>startpt)
			{
				frq[j1]=log10(j);
				real = ireadbuf (fftimg, x,y,0,intp_pref);
				imag = ireadbuf (fftimg, x,y,1,intp_pref);
				amp[j1]=SAFELOG10( sqrt (SQR(real)+SQR(imag)) );
				j1++;
			}
		}

		fit = lsqfit (frq,amp,0,j1-1);
		dp (2, "Macro: Rose plot: angle %d, H=%f, r=%f\n",i,fit.H, fit.r);
		
		angles[k]=i; hurst[k]=0.5*(-fit.H+1);	/* This is really the dimension */

		if (i==0) 
		{
			Havg=Hmin=Hmax=hurst[k]; 
		}
		else
		{
			Hmax = MAX (Hmax, hurst[k]);
			Hmin = MIN (Hmin, hurst[k]);
			Havg += hurst[k];
		}

		mac_user[k+2] = hurst[k];
		k++;
	}

	Havg /= (k-1); Hratio = Hmax/Hmin;
	mac_user[0] = Havg; mac_user[1] = Hratio;
	dp (1, "Macro: Rose: Average Hurst exponent: %f. Anisotropy %f\n",Havg,Hratio);

	free (frq); free (amp);
	return Havg;

}

/*----------- The rose plot GUI --------------------*/

GtkAdjustment *rose_z_adj, *rose_start_adj, *rose_end_adj;

void on_rose_okbutton_clicked  (GtkButton *button, gpointer user_data)
{

	rose_plot (&mainimg, rose_z_adj->value, rose_start_adj->value, rose_end_adj->value, 45);
	cur_slice=0;
	make_imlib_buffer_from_main_image (cur_slice);
}


void on_rose_cancelbutton_clicked  (GtkButton *button, gpointer user_data)
{

	gtk_widget_destroy (GTK_WIDGET(user_data));
	
}




void create_roseplot_wnd (image_type* img)
{
	GtkWidget *rose_wnd;
	GtkWidget *vbox1;
	GtkWidget *label1;
	GtkWidget *table1;
	GtkWidget *rose_z_scale;
	GtkWidget *rose_start_scale;
	GtkWidget *rose_end_scale;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *rose_okbutton;
	GtkWidget *rose_cancelbutton;
	GtkWidget *hseparator1;
	GtkWidget *hseparator2;
	int xmax;
  
	xmax = MIN(img->xmax, img->ymax)/2;
	dim_2d3d=0;
	dim_function=0;

	rose_wnd = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (rose_wnd), "rose_wnd", rose_wnd);
	gtk_container_set_border_width (GTK_CONTAINER (rose_wnd), 3);
	gtk_window_set_title (GTK_WINDOW (rose_wnd), "1/f fractal rose plot");
	gtk_window_set_modal (GTK_WINDOW (rose_wnd), TRUE);

	vbox1 = vbox_in_window (rose_wnd,"dim_vbox1");
	table1 = MakeTable (rose_wnd,vbox1,"dim_table1",2,5);

	label1 = MakeLabel (rose_wnd,table1,"Z slice","label1",0,0);
	rose_z_adj = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, img->zmax-1, 0, 10, 0));
	rose_z_scale = MakeHScale (rose_z_adj,rose_wnd,table1,"dim_lthresh",1,0);
	gtk_scale_set_digits (GTK_SCALE (rose_z_scale), 0);
	
	label2 = MakeLabel (rose_wnd,table1,"Start index","label2",0,1);
	rose_start_adj = GTK_ADJUSTMENT (gtk_adjustment_new (1,1, xmax-1, 1, 10, 0));
	rose_start_scale = MakeHScale (rose_start_adj,rose_wnd,table1,"dim_uthresh",1,1);
	gtk_scale_set_digits (GTK_SCALE (rose_start_scale), 0);
	
	label3 = MakeLabel (rose_wnd,table1,"End index","label3",0,2);
	rose_end_adj = GTK_ADJUSTMENT (gtk_adjustment_new (1, 1, xmax-1, 1, 10, 0));
	rose_end_scale = MakeHScale (rose_end_adj,rose_wnd,table1,"dim_minsize",1,2);
	gtk_scale_set_digits (GTK_SCALE (rose_end_scale), 0);

	hseparator1 = MakeHsep (rose_wnd,table1,"hsep1",0,3);
	hseparator2 = MakeHsep (rose_wnd,table1,"hsep2",1,3);

	rose_okbutton = Button_in_table (rose_wnd,table1,"Compute","dim_okbutton",0,4,on_rose_okbutton_clicked);
	rose_cancelbutton = Button_in_table (rose_wnd,table1,"Dismiss","dim_cancelbutton",1,4,NULL);
	gtk_signal_connect (GTK_OBJECT (rose_cancelbutton), "clicked",
						GTK_SIGNAL_FUNC (on_rose_cancelbutton_clicked), rose_wnd);
	gtk_signal_connect (GTK_OBJECT (rose_wnd), "destroy",
						GTK_SIGNAL_FUNC (on_rose_cancelbutton_clicked), rose_wnd);

	gtk_widget_show (rose_wnd);
}


void roseplot_main ()
{

	create_roseplot_wnd (&mainimg);
	
}




/*****************************************************

	Local Hoelder operator. The Hoelder exponent
	is the local equivalent of the global Hurst
	exponent. 

	NOTE: The collection of Hoelder estimators is
	shoddily done and needs clean-up, particularly
	elimination of functions that don't work right 
*/




/* *****************************************************

	Hoelder exponent through rescaled range analysis:
	Here, we compute the brightness range in a mixed 
	octagonal / square environment and fit a line into
	log range over log radius. This is the new value
	of the central pixel.
	Algorithm described in Russ, pg. 283ff

********************************************************/


char octagon[11][11] = {	{ 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 0},
							{ 0, 0,14,13,11,10,11,13,14, 0, 0},
							{ 0,14,12, 9, 8, 7, 8, 9,12,14, 0},
							{ 0,13, 9, 6, 5, 4, 5, 6, 9,13, 0},
							{ 0,11, 8, 5, 3, 2, 3, 5, 8,11, 0},
							{14,10, 7, 4, 2, 1, 2, 4, 7,10,14},
							{ 0,11, 8, 5, 3, 2, 3, 5, 8,11, 0},
							{ 0,13, 9, 6, 5, 4, 5, 6, 9,13, 0},
							{ 0,14,12, 9, 8, 7, 8, 9,12,14, 0},
							{ 0, 0,14,13,11,10,11,13,14, 0, 0},
							{ 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 0}
						};



float hdists[16] = {0.0, 0.0, 1.0, 1.414, 2.0, 2.236, 2.828, 3.0, 
					3.162, 3.61, 4.0, 4.123, 4.24, 4.472, 5.0, 5.657 };



void hoelder_rra (image_type* img, int slice, int rmax)
{
double xdata[16], ydata[16];
float min[16], max[16];
float *p;
int x,y,x1,y1,radius,maxrad,i,j;
image_type H;
float buf;
fitresult fit;
char nullflg;
char c, sample;

	
	if (allocate_image (&H, B_FLOAT, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();
	p = H.data;
	rmax = MAX (rmax,5);		/* Not worth bothering with any radius < 5 */
	maxrad = MIN(rmax,14);		/* Limit to value 14 of octagon */
	c = 5;						/* offset into center of octagon */

	dp (2, "Hoelder: Neighborhood %dx%d, max radius index %d, Euclidean distance %f\n",rmax,rmax,maxrad,hdists[maxrad]);

	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
			sample=0;
			if ((x==10) && (y==22))
			{
				dp (3, "Breakpoint\n");
				sample=1;
			}

			/*Scan the local neighborhood to get R(r) */

			buf = readbuf_flt (img, x,y,slice);
			for (radius=0; radius<16; radius++) 
			{ 
				min[radius]=buf; max[radius]=buf;	/* Initialize arbitrarily to zero contrast */
				if (sample) dp (1, "Initialize buf to %f\n",buf);
			}
			
			for (j=-c; j<=c; j++)			/* Scan the entire 11x11 octagon */
				for (i=-c; i<=c; i++)
				{
					/* Mirror boundary conditions */
					x1 = x+i; if (x1<0) x1=abs(x1); else if (x1>=img->xmax) x1=(2*img->xmax)-x1-1;
					y1 = y+j; if (y1<0) y1=abs(y1); else if (y1>=img->ymax) y1=(2*img->ymax)-y1-1;
				
					buf = readbuf_flt (img, x1, y1, slice);
					if (sample) dp (1, "Testing value at %d,%d: %f\n",x1,y1,buf);
					for (radius=1; radius<=maxrad; radius++)
						if ((octagon[i+c][j+c]) && (octagon[i+c][j+c] <= radius))
						{
							if (buf > max[radius-1])
							{
								max[radius-1]=buf;
								if (sample) dp (1, "Updated max at radius %d\n", radius-1);
							}
							if (buf < min[radius-1])
							{
								min[radius-1]=buf;
								if (sample) dp (1, "Updated min at radius %d\n", radius-1);
							}
						}
				}
				
			/* end getting local neighborhood range, compute Hoelder exp */

			if (sample) dp (1, "***** Here is the table for point at %d,%d:\n",x,y);
			nullflg=0;
			for (radius=2; radius<=maxrad; radius++)
			{
				if (sample) dp(1,"Radius=%d, EucliDist=%f, Range=%f\n",radius-2,hdists[radius],max[radius-1] - min[radius-1]);
				/* Range over neighborhood radius, R(r) */
				xdata[radius-2] = SAFELOG10(hdists[radius]);
				ydata[radius-2] = SAFELOG10(max[radius-1] - min[radius-1]);
				if ((max[radius-1] - min[radius-1]) ==0.0) nullflg=1;	/* log of zero */
			}
			if (!nullflg)
			{
				fit = lsqfit (xdata, ydata, 0, maxrad-2);
				if (fabs(fit.r) > 0.6)		/* Reject bad fits - we want only true fractals */
					p[x + H.xmax*y] = fabs(fit.H);
				if (sample) dp (1, "***** This gives us H=%f and Rsqr=%f\n",fit.H, fit.r);
			}
		}

	handback (img, &H);

}


/*****************************************************

	Local Hoelder operator computed by mass dimension.
	We compute the grayscale mass in a square region
	around each pixel

********************************************************/




void hoelder_massdim (image_type* img, int slice, int rmax)
{
double xdata[32], ydata[32];
float *p;
int x,y,x1,y1,i,j,k,n;
image_type H;
float buf,mass;
fitresult fit;
char c, sample;

	
	if (allocate_image (&H, B_FLOAT, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();
	p = H.data;
	rmax = MAX (rmax,5); 	/* Demand minimum outer radius of 5 to get at least _some_ data */


	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
			sample=0;
			if ((x==10) && (y==22))
			{
				dp (3, "Breakpoint\n");
				sample=1;
			}
			
			/*Scan the local neighborhood to get m(r) */

			k=0;
			for (c=1; c<=rmax; c++)			/* increasing radius */
			{
				mass=0; n=0;
				for (j=-c; j<=c; j++)
					for (i=-c; i<=c; i++)
					{
						/* Mirror boundary conditions */
						x1 = x+i; if (x1<0) x1=abs(x1); else if (x1>=img->xmax) x1=(2*img->xmax)-x1-1;
						y1 = y+j; if (y1<0) y1=abs(y1); else if (y1>=img->ymax) y1=(2*img->ymax)-y1-1;
					
						buf = readbuf_flt (img, x1, y1, slice);
						mass += buf; n++;
					}

				if ((mass>0) && (k<31))
				{
					ydata[k] = SAFELOG10(mass);
					xdata[k] = SAFELOG10(2*c+1);
					k++;
				}
			}
				
			/* end getting local neighborhood range, compute Hoelder exp */

			if (k>3)
			{
				fit = lsqfit (xdata, ydata, 0, k-1);
				if (fabs(fit.r) > 0.6)		/* Reject bad fits - we want only true fractals */
					p[x + H.xmax*y] = fabs(fit.H);
			}
		}

	handback (img, &H);

}


/* Similar function, to be coupled to a button. Determines
   the local Hoelder exponent for the location that was last
   clicked on (clickstack)
*/


float hoelder_massdim_click (image_type* img, int x, int y, int z, int rmax)
{
double xdata[32], ydata[32];
int x1,y1,i,j,k,n;
float buf,mass;
fitresult fit;
char c;
char s[1024];

	rmax = MAX (rmax,5); 

	/*Scan the local neighborhood to get m(r) */

	k=0;
	for (c=1; c<=rmax; c++)			/* increasing radius */
	{
		mass=0; n=0;
		for (j=-c; j<=c; j++)
			for (i=-c; i<=c; i++)
			{
				/* Mirror boundary conditions */
				x1 = x+i; if (x1<0) x1=abs(x1); else if (x1>=img->xmax) x1=(2*img->xmax)-x1-1;
				y1 = y+j; if (y1<0) y1=abs(y1); else if (y1>=img->ymax) y1=(2*img->ymax)-y1-1;
					
				buf = readbuf_flt (img, x1, y1, z);
				mass += buf; n++;
			}

		dp (1, "Local mass dimension: Iteration %d: Square size %d, area %d; mass %8.1f\n",c,2*c+1,n, mass);
		
		
		if (mass>0)
		{
			ydata[k] = SAFELOG10(mass);
			xdata[k] = SAFELOG10(2*c+1);
			k++;
		}
	}
			
	/* end getting local neighborhood range, compute Hoelder exp */

	if (k>3)
	{
		dp (1, "Local mass dimension: This is the log data I fit:\n");
		for (i=0; i<k; i++)
			dp (1,"        %d: log radius %4.2f, log mass: %4.2f\n",i,xdata[i],ydata[i]);
		fit = lsqfit (xdata, ydata, 0, k-1);
		sprintf (s, "Hoelder at %d,%d,%d: H=%4.2f, r=%4.2f",x,y,z,fit.H,fit.r);
	}
	else
		sprintf (s, "Holeder: Insufficient data for fit");
	
	statusbar_display (s);
	dp (1, "Local mass dimension: %s\n",s);
	return fit.H;
}


void hoelder_massdim_clickif ()
{
	
	hoelder_massdim_click (&mainimg, clickstack[clickstack_last-1].x, clickstack[clickstack_last-1].y, clickstack[clickstack_last-1].z, 9);
	
}






/**********************************************************

	Local Hoelder operator computed by dispersion analysis.
	Dispersion analysis looks at the effect of averaging
	neighboring points. The SD of each time series with
	increasing bin size is drawn over the bin size,
	and a log-log line is fitted. The slope of this line 
	equals H-1.
	DISPERSION ANALYSIS WORKS ONLY ON FBN, NEVER ON FBM

*************************************************************/




void hoelder_dispersion (image_type* img, int slice, int rmax)
{
double xdata[16], ydata[16];
float *p;
int x,y,x1,y1,i,j,k;
image_type H;
float buf,mass;
fitresult fit;
char c;

	
	if (allocate_image (&H, B_FLOAT, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();
	p = H.data;


	for (y=0; y<img->ymax; y++)
		for (x=0; x<img->xmax; x++)
		{
			/*Scan the local neighborhood to get m(r) */

			k=0; mass=0;
			for (c=1; c<=rmax; c++)			/* increasing radius */
			{
				for (j=-c; j<=c; j++)
					for (i=-c; i<=c; i++)
					{
						/* Mirror boundary conditions */
						x1 = x+i; if (x1<0) x1=abs(x1); else if (x1>=img->xmax) x1=(2*img->xmax)-x1-1;
						y1 = y+j; if (y1<0) y1=abs(y1); else if (y1>=img->ymax) y1=(2*img->ymax)-y1-1;
					
						buf = readbuf_flt (img, x1, y1, slice);
						mass += buf;
					}

				if (mass>0)
				{
					ydata[k] = SAFELOG10(mass);
					xdata[k] = SAFELOG10(c);
					k++;
				}
			}
				
			/* end getting local neighborhood range, compute Hoelder exp */

			if (k>3)
			{
				fit = lsqfit (xdata, ydata, 0, k-1);
				if (fabs(fit.r) > 0.6)		/* Reject bad fits - we want only true fractals */
					p[x + H.xmax*y] = fabs(fit.H);
			}
		}

	handback (img, &H);

}






void local_hoelder_main (int nghsize)
{
	undo_nexus (&mainimg);

//	hoelder (&mainimg, cur_slice, nghsize);
	hoelder_massdim (&mainimg, cur_slice, nghsize+5);
	cur_slice=0;
	make_imlib_buffer_from_main_image (cur_slice);
}





/******************************************************

	Another quick hack. Perform the iterative step
	that leads to the Sierpinski Carpet.
	Connect this to one of the macro buttons.
	
*******************************************************/



void sierp_carpet (image_type* img, int z, float thresh)
{
int x,y,x1,y1,x2,y2;
image_type carpet;
short* p;
float buf;
short bit;
long idx;


	if (allocate_image (&carpet, B_SHORT, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();
	p = carpet.data;
	
	
	y2=0;
	for (y=0; y<img->ymax; y+=3)
	{
		x2=0;
		for (x=0; x<img->xmax; x+=3)
		{
			bit=0;
			for (y1=0; y1<3; y1++)
				for (x1=0; x1<3; x1++)
				{
					buf = readbuf_flt (img,x+x1,y+y1,z);
					if (buf>thresh) bit=1;
				}
			
			/* create 8 copies */
			
			if (bit)
				for (y1=0; y1<3; y1++)
					for (x1=0; x1<3; x1++)
						if ((x1!=1) || (y1!=1))
						{
							idx = x2 + (x1*(img->xmax+1))/3 + img->xmax* (y2 + (y1*(img->ymax+1))/3);
							if (idx < img->xmax*img->ymax) p[idx] = 255;
						}		
			
			x2++;
		}
		y2++;
	}

	handback (img, &carpet);

}





void sierp_iterator (image_type* img, int z, float thresh)
{
int x,y,x1,y1,x2,y2,xm,ym;
image_type gasket;
short* p;
float buf;
short bit;


	if (allocate_image (&gasket, B_SHORT, img->xmax, img->ymax, 1)==-1)
		fatal_memerr();
	p = gasket.data;
	
	
	y2=0; xm=img->xmax; ym = img->ymax;
	for (y=0; y<ym-1; y+=2)
	{
		x2=0;
		for (x=0; x<xm-1; x+=2)
		{
			bit=0;
			for (y1=0; y1<2; y1++)
				for (x1=0; x1<2; x1++)
				{
					buf = readbuf_flt (img,x+x1,y+y1,z);
					if (buf>thresh) bit++;
				}
			
			/* create 3 copies */
			
			if (bit)
			{
				p[ xm/4+x2 + xm*y2 ] = 255;				/* top center */
				p[ x2 + xm*(ym/2+y2) ] = 255;  			/* bottom left */
				p[ xm/2+x2 + xm*(ym/2+y2) ] = 255; 		/* bottom right */
			}
			
			x2++;
		}
		y2++;
	}

	handback (img, &gasket);

}
	

